@@ -21,7 +21,6 @@
 import re
 import sys
 import os
-import locale
 import pkgutil
 import logging
 
@@ -70,9 +69,13 @@ def print_help(available_plugins=None):
 
 
 def preparse_plugin(module_name, module_dir):
-    """simple regex based parsing for plugin scripts,
-       so we don't have to import them when we just need the summary,
-       but not plan on executing this particular plugin."""
+    """No summary.
+
+    Simple regex based parsing for plugin scripts.
+
+    So we don't have to import them when we just need the summary,
+    but not plan on executing this particular plugin.
+    """
     if '.' in module_name:
         raise ValueError("No '.' allowed in fdroid plugin modules: '{}'"
                          .format(module_name))
@@ -198,7 +201,7 @@ def main():
     else:
         mod = __import__(available_plugins[command]['name'], None, None, [command])
 
-    system_langcode, system_encoding = locale.getdefaultlocale()
+    system_encoding = sys.getdefaultencoding()
     if system_encoding is None or system_encoding.lower() not in ('utf-8', 'utf8'):
         logging.warning(_("Encoding is set to '{enc}' fdroid might run "
                           "into encoding issues. Please set it to 'UTF-8' "
