@@ -23,8 +23,6 @@ import glob
 import subprocess
 import posixpath
 import re
-import resource
-import sys
 import tarfile
 import threading
 import traceback
@@ -55,14 +53,19 @@ except ImportError:
 def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
     """Do a build on the builder vm.
 
-    :param app: app metadata dict
-    :param build:
-    :param vcs: version control system controller object
-    :param build_dir: local source-code checkout of app
-    :param output_dir: target folder for the build result
-    :param force:
+    Parameters
+    ----------
+    app
+        app metadata dict
+    build
+    vcs
+        version control system controller object
+    build_dir
+        local source-code checkout of app
+    output_dir
+        target folder for the build result
+    force
     """
-
     global buildserverid
 
     try:
@@ -78,12 +81,11 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
 
     output = None
     try:
-        if not buildserverid:  # pylint: disable=used-before-assignment
+        if not buildserverid:
             try:
                 buildserverid = subprocess.check_output(['vagrant', 'ssh', '-c',
                                                          'cat /home/vagrant/buildserverid'],
                                                         cwd='builder').strip().decode()
-                status_output['buildserverid'] = buildserverid
                 logging.debug(_('Fetched buildserverid from VM: {buildserverid}')
                               .format(buildserverid=buildserverid))
             except Exception as e:
@@ -238,7 +240,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         try:
             cmd_stdout = chan.makefile('rb', 1024)
             output = bytes()
-            output += common.get_android_tools_version_log(build.ndk_path()).encode()
+            output += common.get_android_tools_version_log().encode()
             while not chan.exit_status_ready():
                 line = cmd_stdout.readline()
                 if line:
@@ -263,7 +265,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             else:
                 message = "Build.py failed on server for {0}:{1}"
             raise BuildException(message.format(app.id, build.versionName),
-                                 None if options.verbose else str(output, 'utf-8'))
+                                 str(output, 'utf-8'))
 
         # Retreive logs...
         toolsversion_log = common.get_toolsversion_logname(app, build)
@@ -289,7 +291,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         except Exception:
             raise BuildException(
                 "Build failed for {0}:{1} - missing output files".format(
-                    app.id, build.versionName), None if options.verbose else str(output, 'utf-8'))
+                    app.id, build.versionName), str(output, 'utf-8'))
         ftp.close()
 
     finally:
@@ -321,7 +323,7 @@ def force_gradle_build_tools(build_dir, build_tools):
 
 
 def transform_first_char(string, method):
-    """Uses method() on the first character of string."""
+    """Use method() on the first character of string."""
     if len(string) == 0:
         return string
     if len(string) == 1:
@@ -334,11 +336,10 @@ def add_failed_builds_entry(failed_builds, appid, build, entry):
 
 
 def get_metadata_from_apk(app, build, apkfile):
-    """get the required metadata from the built APK
+    """Get the required metadata from the built APK.
 
-    versionName is allowed to be a blank string, i.e. ''
+    VersionName is allowed to be a blank string, i.e. ''
     """
-
     appid, versionCode, versionName = common.get_apk_id(apkfile)
     native_code = common.get_native_code(apkfile)
 
@@ -362,13 +363,16 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
     ndk_path = build.ndk_path()
     if build.ndk or (build.buildjni and build.buildjni != ['no']):
         if not ndk_path:
-            logging.critical("Android NDK version '%s' could not be found!" % build.ndk or 'r12b')
-            logging.critical("Configured versions:")
+            logging.warning("Android NDK version '%s' could not be found!" % build.ndk)
+            logging.warning("Configured versions:")
             for k, v in config['ndk_paths'].items():
                 if k.endswith("_orig"):
                     continue
-                logging.critical("  %s: %s" % (k, v))
-            raise FDroidException()
+                logging.warning("  %s: %s" % (k, v))
+            if onserver:
+                common.auto_install_ndk(build)
+            else:
+                raise FDroidException()
         elif not os.path.isdir(ndk_path):
             logging.critical("Android NDK '%s' is not a directory!" % ndk_path)
             raise FDroidException()
@@ -400,7 +404,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
         log_path = os.path.join(log_dir,
                                 common.get_toolsversion_logname(app, build))
         with open(log_path, 'w') as f:
-            f.write(common.get_android_tools_version_log(build.ndk_path()))
+            f.write(common.get_android_tools_version_log())
     else:
         if build.sudo:
             logging.warning('%s:%s runs this on the buildserver with sudo:\n\t%s\nThese commands were skipped because fdroid build is not running on a dedicated build server.'
@@ -805,11 +809,16 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
                                     str(build.versionCode)))
         if (options.scan_binary or config.get('scan_binary')) and not options.skipscan:
             if scanner.scan_binary(src):
-                raise BuildException("Found blacklisted packages in final apk!")
+                raise BuildException("Found blocklisted packages in final apk!")
 
     # Copy the unsigned APK to our destination directory for further
     # processing (by publish.py)...
-    dest = os.path.join(output_dir, common.get_release_filename(app, build))
+    dest = os.path.join(
+        output_dir,
+        common.get_release_filename(
+            app, build, common.get_file_extension(src)
+        )
+    )
     shutil.copyfile(src, dest)
 
     # Move the source tarball into the output directory...
@@ -821,23 +830,29 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
 def trybuild(app, build, build_dir, output_dir, log_dir, also_check_dir,
              srclib_dir, extlib_dir, tmp_dir, repo_dir, vcs, test,
              server, force, onserver, refresh):
+    """Build a particular version of an application, if it needs building.
+
+    Parameters
+    ----------
+    output_dir
+        The directory where the build output will go.
+        Usually this is the 'unsigned' directory.
+    repo_dir
+        The repo directory - used for checking if the build is necessary.
+    also_check_dir
+        An additional location for checking if the build
+        is necessary (usually the archive repo)
+    test
+        True if building in test mode, in which case the build will
+        always happen, even if the output already exists. In test mode, the
+        output directory should be a temporary location, not any of the real
+        ones.
+
+    Returns
+    -------
+    Boolean
+        True if the build was done, False if it wasn't necessary.
     """
-    Build a particular version of an application, if it needs building.
-
-    :param output_dir: The directory where the build output will go. Usually
-       this is the 'unsigned' directory.
-    :param repo_dir: The repo directory - used for checking if the build is
-       necessary.
-    :param also_check_dir: An additional location for checking if the build
-       is necessary (usually the archive repo)
-    :param test: True if building in test mode, in which case the build will
-       always happen, even if the output already exists. In test mode, the
-       output directory should be a temporary location, not any of the real
-       ones.
-
-    :returns: True if the build was done, False if it wasn't necessary.
-    """
-
     dest_file = common.get_release_filename(app, build)
 
     dest = os.path.join(output_dir, dest_file)
@@ -863,6 +878,10 @@ def trybuild(app, build, build_dir, output_dir, log_dir, also_check_dir,
         # grabbing the source now.
         vcs.gotorevision(build.commit, refresh)
 
+        # Initialise submodules if required
+        if build.submodules:
+            vcs.initsubmodules()
+
         build_server(app, build, vcs, build_dir, output_dir, log_dir, force)
     else:
         build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, extlib_dir, tmp_dir, force, onserver, refresh)
@@ -870,7 +889,7 @@ def trybuild(app, build, build_dir, output_dir, log_dir, also_check_dir,
 
 
 def force_halt_build(timeout):
-    """Halt the currently running Vagrant VM, to be called from a Timer"""
+    """Halt the currently running Vagrant VM, to be called from a Timer."""
     logging.error(_('Force halting build after {0} sec timeout!').format(timeout))
     timeout_event.set()
     vm = vmtools.get_build_vm('builder')
@@ -878,8 +897,13 @@ def force_halt_build(timeout):
 
 
 def parse_commandline():
-    """Parse the command line. Returns options, parser."""
+    """Parse the command line.
 
+    Returns
+    -------
+    options
+    parser
+    """
     parser = argparse.ArgumentParser(usage="%(prog)s [options] [APPID[:VERCODE] [APPID[:VERCODE] ...]]")
     common.setup_global_opts(parser)
     parser.add_argument("appid", nargs='*', help=_("application ID with optional versionCode in the form APPID[:VERCODE]"))
@@ -910,7 +934,7 @@ def parse_commandline():
     parser.add_argument("-a", "--all", action="store_true", default=False,
                         help=_("Build all applications available"))
     parser.add_argument("-w", "--wiki", default=False, action="store_true",
-                        help=_("Update the wiki"))
+                        help=argparse.SUPPRESS)
     metadata.add_metadata_arguments(parser)
     options = parser.parse_args()
     metadata.warnings_action = options.W
@@ -1013,7 +1037,7 @@ def main():
 
     # Read all app and srclib metadata
     pkgs = common.read_pkg_args(options.appid, True)
-    allapps = metadata.read_metadata(pkgs, options.refresh, sort_by_time=True, check_vcs=True)
+    allapps = metadata.read_metadata(pkgs, sort_by_time=True)
     apps = common.read_app_args(options.appid, allapps, True)
 
     for appid, app in list(apps.items()):
@@ -1024,17 +1048,22 @@ def main():
         raise FDroidException("No apps to process.")
 
     # make sure enough open files are allowed to process everything
-    soft, hard = resource.getrlimit(resource.RLIMIT_NOFILE)
-    if len(apps) > soft:
-        try:
-            soft = len(apps) * 2
-            if soft > hard:
-                soft = hard
-            resource.setrlimit(resource.RLIMIT_NOFILE, (soft, hard))
-            logging.debug(_('Set open file limit to {integer}')
-                          .format(integer=soft))
-        except (OSError, ValueError) as e:
-            logging.warning(_('Setting open file limit failed: ') + str(e))
+    try:
+        import resource  # not available on Windows
+
+        soft, hard = resource.getrlimit(resource.RLIMIT_NOFILE)
+        if len(apps) > soft:
+            try:
+                soft = len(apps) * 2
+                if soft > hard:
+                    soft = hard
+                resource.setrlimit(resource.RLIMIT_NOFILE, (soft, hard))
+                logging.debug(_('Set open file limit to {integer}')
+                              .format(integer=soft))
+            except (OSError, ValueError) as e:
+                logging.warning(_('Setting open file limit failed: ') + str(e))
+    except ImportError:
+        pass
 
     if options.latest:
         for app in apps.values():
@@ -1044,19 +1073,15 @@ def main():
                 app['Builds'] = [build]
                 break
 
-    if options.wiki:
-        import mwclient
-        site = mwclient.Site((config['wiki_protocol'], config['wiki_server']),
-                             path=config['wiki_path'])
-        site.login(config['wiki_user'], config['wiki_password'])
-
     # Build applications...
     failed_builds = []
     build_succeeded = []
+    build_succeeded_ids = []
     status_output['failedBuilds'] = failed_builds
     status_output['successfulBuilds'] = build_succeeded
-    # Only build for 36 hours, then stop gracefully.
-    endtime = time.time() + 36 * 60 * 60
+    status_output['successfulBuildIds'] = build_succeeded_ids
+    # Only build for 72 hours, then stop gracefully.
+    endtime = time.time() + 72 * 60 * 60
     max_build_time_reached = False
     for appid, app in apps.items():
 
@@ -1080,11 +1105,9 @@ def main():
             else:
                 timer = None
 
-            wikilog = None
-            build_starttime = common.get_wiki_timestamp()
             tools_version_log = ''
             if not options.onserver:
-                tools_version_log = common.get_android_tools_version_log(build.ndk_path())
+                tools_version_log = common.get_android_tools_version_log()
                 common.write_running_status_json(status_output)
             try:
 
@@ -1125,7 +1148,7 @@ def main():
                         url = url.replace('%v', build.versionName)
                         url = url.replace('%c', str(build.versionCode))
                         logging.info("...retrieving " + url)
-                        of = re.sub(r'.apk$', '.binary.apk', common.get_release_filename(app, build))
+                        of = re.sub(r'\.apk$', '.binary.apk', common.get_release_filename(app, build))
                         of = os.path.join(binaries_dir, of)
                         try:
                             net.download_file(url, local_filename=of)
@@ -1145,8 +1168,12 @@ def main():
                             compare_result = \
                                 common.verify_apks(of, unsigned_apk, tmpdir)
                             if compare_result:
-                                logging.debug('removing %s', unsigned_apk)
-                                os.remove(unsigned_apk)
+                                if options.test:
+                                    logging.warning(_('Keeping failed build "{apkfilename}"')
+                                                    .format(apkfilename=unsigned_apk))
+                                else:
+                                    logging.debug('removing %s', unsigned_apk)
+                                    os.remove(unsigned_apk)
                                 logging.debug('removing %s', of)
                                 os.remove(of)
                                 compare_result = compare_result.split('\n')
@@ -1168,7 +1195,7 @@ def main():
                                              'successfully')
 
                     build_succeeded.append(app)
-                    wikilog = "Build succeeded"
+                    build_succeeded_ids.append([app['id'], build.versionCode])
 
             except VCSException as vcse:
                 reason = str(vcse).split('\n', 1)[0] if options.verbose else str(vcse)
@@ -1178,14 +1205,17 @@ def main():
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
                 add_failed_builds_entry(failed_builds, appid, build, vcse)
-                wikilog = str(vcse)
+                common.deploy_build_log_with_rsync(
+                    appid, build.versionCode, "".join(traceback.format_exc())
+                )
             except FDroidException as e:
+                tstamp = time.strftime("%Y-%m-%d %H:%M:%SZ", time.gmtime())
                 with open(os.path.join(log_dir, appid + '.log'), 'a+') as f:
                     f.write('\n\n============================================================\n')
                     f.write('versionCode: %s\nversionName: %s\ncommit: %s\n' %
                             (build.versionCode, build.versionName, build.commit))
                     f.write('Build completed at '
-                            + common.get_wiki_timestamp() + '\n')
+                            + tstamp + '\n')
                     f.write('\n' + tools_version_log + '\n')
                     f.write(str(e))
                 logging.error("Could not build app %s: %s" % (appid, e))
@@ -1193,7 +1223,9 @@ def main():
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
                 add_failed_builds_entry(failed_builds, appid, build, e)
-                wikilog = e.get_wikitext()
+                common.deploy_build_log_with_rsync(
+                    appid, build.versionCode, "".join(traceback.format_exc())
+                )
             except Exception as e:
                 logging.error("Could not build app %s due to unknown error: %s" % (
                     appid, traceback.format_exc()))
@@ -1201,32 +1233,9 @@ def main():
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
                 add_failed_builds_entry(failed_builds, appid, build, e)
-                wikilog = str(e)
-
-            if options.wiki and wikilog:
-                try:
-                    # Write a page with the last build log for this version code
-                    lastbuildpage = appid + '/lastbuild_' + build.versionCode
-                    newpage = site.Pages[lastbuildpage]
-                    with open(os.path.join('tmp', 'fdroidserverid')) as fp:
-                        fdroidserverid = fp.read().rstrip()
-                    txt = "* build session started at " + common.get_wiki_timestamp(start_timestamp) + '\n' \
-                          + "* this build started at " + build_starttime + '\n' \
-                          + "* this build completed at " + common.get_wiki_timestamp() + '\n' \
-                          + common.get_git_describe_link() \
-                          + '* fdroidserverid: [https://gitlab.com/fdroid/fdroidserver/commit/' \
-                          + fdroidserverid + ' ' + fdroidserverid + ']\n\n'
-                    if buildserverid:
-                        txt += '* buildserverid: [https://gitlab.com/fdroid/fdroidserver/commit/' \
-                               + buildserverid + ' ' + buildserverid + ']\n\n'
-                    txt += tools_version_log + '\n\n'
-                    txt += '== Build Log ==\n\n' + wikilog
-                    newpage.save(txt, summary='Build log')
-                    # Redirect from /lastbuild to the most recent build log
-                    newpage = site.Pages[appid + '/lastbuild']
-                    newpage.save('#REDIRECT [[' + lastbuildpage + ']]', summary='Update redirect')
-                except Exception as e:
-                    logging.error("Error while attempting to publish build log: %s" % e)
+                common.deploy_build_log_with_rsync(
+                    appid, build.versionCode, "".join(traceback.format_exc())
+                )
 
             if timer:
                 timer.cancel()  # kill the watchdog timer
@@ -1251,27 +1260,15 @@ def main():
         logging.info(ngettext("{} build failed",
                               "{} builds failed", len(failed_builds)).format(len(failed_builds)))
 
-    if options.wiki:
-        wiki_page_path = 'build_' + time.strftime('%s', start_timestamp)
-        newpage = site.Pages[wiki_page_path]
-        txt = ''
-        txt += "* command line: <code>%s</code>\n" % ' '.join(sys.argv)
-        txt += "* started at %s\n" % common.get_wiki_timestamp(start_timestamp)
-        txt += "* completed at %s\n" % common.get_wiki_timestamp()
-        if buildserverid:
-            txt += ('* buildserverid: [https://gitlab.com/fdroid/fdroidserver/commit/{id} {id}]\n'
-                    .format(id=buildserverid))
-        if fdroidserverid:
-            txt += ('* fdroidserverid: [https://gitlab.com/fdroid/fdroidserver/commit/{id} {id}]\n'
-                    .format(id=fdroidserverid))
+    if options.server:
         if os.cpu_count():
-            txt += "* host processors: %d\n" % os.cpu_count()
+            status_output['hostOsCpuCount'] = os.cpu_count()
         if os.path.isfile('/proc/meminfo') and os.access('/proc/meminfo', os.R_OK):
             with open('/proc/meminfo') as fp:
                 for line in fp:
                     m = re.search(r'MemTotal:\s*([0-9].*)', line)
                     if m:
-                        txt += "* host RAM: %s\n" % m.group(1)
+                        status_output['hostProcMeminfoMemTotal'] = m.group(1)
                         break
         fdroid_path = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))
         buildserver_config = os.path.join(fdroid_path, 'makebuildserver.config.py')
@@ -1280,16 +1277,13 @@ def main():
                 for line in configfile:
                     m = re.search(r'cpus\s*=\s*([0-9].*)', line)
                     if m:
-                        txt += "* guest processors: %s\n" % m.group(1)
+                        status_output['guestVagrantVmCpus'] = m.group(1)
                     m = re.search(r'memory\s*=\s*([0-9].*)', line)
                     if m:
-                        txt += "* guest RAM: %s MB\n" % m.group(1)
-        txt += "* successful builds: %d\n" % len(build_succeeded)
-        txt += "* failed builds: %d\n" % len(failed_builds)
-        txt += "\n\n"
-        newpage.save(txt, summary='Run log')
-        newpage = site.Pages['build']
-        newpage.save('#REDIRECT [[' + wiki_page_path + ']]', summary='Update redirect')
+                        status_output['guestVagrantVmMemory'] = m.group(1)
+
+    if buildserverid:
+        status_output['buildserver'] = {'commitId': buildserverid}
 
     if not options.onserver:
         common.write_status_json(status_output)
