@@ -31,6 +31,7 @@ from distutils.version import LooseVersion
 import logging
 import copy
 import urllib.parse
+from pathlib import Path
 
 from . import _
 from . import common
@@ -44,247 +45,254 @@ from .exception import VCSException, NoSubmodulesException, FDroidException, Met
 # required.
 def check_http(app):
 
-    ignoreversions = app.UpdateCheckIgnore
-    ignoresearch = re.compile(ignoreversions).search if ignoreversions else None
+    if not app.UpdateCheckData:
+        raise FDroidException('Missing Update Check Data')
 
-    try:
-
-        if not app.UpdateCheckData:
-            raise FDroidException('Missing Update Check Data')
-
-        urlcode, codeex, urlver, verex = app.UpdateCheckData.split('|')
-        parsed = urllib.parse.urlparse(urlcode)
+    urlcode, codeex, urlver, verex = app.UpdateCheckData.split('|')
+    parsed = urllib.parse.urlparse(urlcode)
+    if not parsed.netloc or not parsed.scheme or parsed.scheme != 'https':
+        raise FDroidException(_('UpdateCheckData has invalid URL: {url}').format(url=urlcode))
+    if urlver != '.':
+        parsed = urllib.parse.urlparse(urlver)
         if not parsed.netloc or not parsed.scheme or parsed.scheme != 'https':
             raise FDroidException(_('UpdateCheckData has invalid URL: {url}').format(url=urlcode))
-        if urlver != '.':
-            parsed = urllib.parse.urlparse(urlver)
-            if not parsed.netloc or not parsed.scheme or parsed.scheme != 'https':
-                raise FDroidException(_('UpdateCheckData has invalid URL: {url}').format(url=urlcode))
 
-        vercode = None
-        if len(urlcode) > 0:
-            logging.debug("...requesting {0}".format(urlcode))
-            req = urllib.request.Request(urlcode, None, headers=net.HEADERS)
-            resp = urllib.request.urlopen(req, None, 20)  # nosec B310 scheme is filtered above
-            page = resp.read().decode('utf-8')
+    logging.debug("...requesting {0}".format(urlcode))
+    req = urllib.request.Request(urlcode, None, headers=net.HEADERS)
+    resp = urllib.request.urlopen(req, None, 20)  # nosec B310 scheme is filtered above
+    page = resp.read().decode('utf-8')
 
-            m = re.search(codeex, page)
-            if not m:
-                raise FDroidException("No RE match for version code")
-            vercode = m.group(1).strip()
+    m = re.search(codeex, page)
+    if not m:
+        raise FDroidException("No RE match for version code")
+    vercode = m.group(1).strip()
 
-        version = "??"
-        if len(urlver) > 0:
-            if urlver != '.':
-                logging.debug("...requesting {0}".format(urlver))
-                req = urllib.request.Request(urlver, None)
-                resp = urllib.request.urlopen(req, None, 20)  # nosec B310 scheme is filtered above
-                page = resp.read().decode('utf-8')
+    if urlver != '.':
+        logging.debug("...requesting {0}".format(urlver))
+        req = urllib.request.Request(urlver, None)
+        resp = urllib.request.urlopen(req, None, 20)  # nosec B310 scheme is filtered above
+        page = resp.read().decode('utf-8')
 
-            m = re.search(verex, page)
-            if not m:
-                raise FDroidException("No RE match for version")
-            version = m.group(1)
+    m = re.search(verex, page)
+    if not m:
+        raise FDroidException("No RE match for version")
+    version = m.group(1)
 
-        if ignoresearch and version:
-            if not ignoresearch(version):
-                return (version, vercode)
-            else:
-                return (None, ("Version {version} is ignored").format(version=version))
-        else:
-            return (version, vercode)
-    except FDroidException:
-        msg = "Could not complete http check for app {0} due to unknown error: {1}".format(app.id, traceback.format_exc())
-        return (None, msg)
+    if app.UpdateCheckIgnore and re.search(app.UpdateCheckIgnore, version):
+        logging.info("Version {version} for {appid} is ignored".format(version=version, appid=app.id))
+        return (None, None)
+
+    return (version, vercode)
 
 
-# Check for a new version by looking at the tags in the source repo.
-# Whether this can be used reliably or not depends on
-# the development procedures used by the project's developers. Use it with
-# caution, because it's inappropriate for many projects.
-# Returns (None, "a message") if this didn't work, or (version, vercode, tag) for
-# the details of the current version.
 def check_tags(app, pattern):
+    """Check for a new version by looking at the tags in the source repo.
 
-    try:
+    Whether this can be used reliably or not depends on
+    the development procedures used by the project's developers. Use it with
+    caution, because it's inappropriate for many projects.
+    """
+    if app.RepoType == 'srclib':
+        build_dir = Path('build/srclib') / app.Repo
+        repotype = common.getsrclibvcs(app.Repo)
+    else:
+        build_dir = Path('build') / app.id
+        repotype = app.RepoType
 
-        if app.RepoType == 'srclib':
-            build_dir = os.path.join('build', 'srclib', app.Repo)
-            repotype = common.getsrclibvcs(app.Repo)
-        else:
-            build_dir = os.path.join('build', app.id)
-            repotype = app.RepoType
+    if repotype not in ('git', 'git-svn', 'hg', 'bzr'):
+        raise MetaDataException(_('Tags update mode only works for git, hg, bzr and git-svn repositories currently'))
 
-        if repotype not in ('git', 'git-svn', 'hg', 'bzr'):
-            return (None, 'Tags update mode only works for git, hg, bzr and git-svn repositories currently', None)
+    if repotype == 'git-svn' and ';' not in app.Repo:
+        raise MetaDataException(_('Tags update mode used in git-svn, but the repo was not set up with tags'))
 
-        if repotype == 'git-svn' and ';' not in app.Repo:
-            return (None, 'Tags update mode used in git-svn, but the repo was not set up with tags', None)
+    # Set up vcs interface and make sure we have the latest code...
+    vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
 
-        # Set up vcs interface and make sure we have the latest code...
-        vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
+    vcs.gotorevision(None)
 
-        vcs.gotorevision(None)
+    last_build = app.get_last_build()
 
-        last_build = app.get_last_build()
+    try_init_submodules(app, last_build, vcs)
 
-        try_init_submodules(app, last_build, vcs)
+    htag = None
+    hver = None
+    hcode = "0"
 
-        hpak = None
-        htag = None
-        hver = None
-        hcode = "0"
+    tags = []
+    if repotype == 'git':
+        tags = vcs.latesttags()
+    else:
+        tags = vcs.gettags()
+    if not tags:
+        raise FDroidException(_('No tags found'))
 
-        tags = []
-        if repotype == 'git':
-            tags = vcs.latesttags()
-        else:
-            tags = vcs.gettags()
+    logging.debug("All tags: " + ','.join(tags))
+    if pattern:
+        pat = re.compile(pattern)
+        tags = [tag for tag in tags if pat.match(tag)]
         if not tags:
-            return (None, "No tags found", None)
+            raise FDroidException(_('No matching tags found'))
+        logging.debug("Matching tags: " + ','.join(tags))
 
-        logging.debug("All tags: " + ','.join(tags))
-        if pattern:
-            pat = re.compile(pattern)
-            tags = [tag for tag in tags if pat.match(tag)]
-            if not tags:
-                return (None, "No matching tags found", None)
-            logging.debug("Matching tags: " + ','.join(tags))
+    if len(tags) > 5 and repotype == 'git':
+        tags = tags[:5]
+        logging.debug("Latest tags: " + ','.join(tags))
 
-        if len(tags) > 5 and repotype == 'git':
-            tags = tags[:5]
-            logging.debug("Latest tags: " + ','.join(tags))
+    for tag in tags:
+        logging.debug("Check tag: '{0}'".format(tag))
+        vcs.gotorevision(tag)
 
-        for tag in tags:
-            logging.debug("Check tag: '{0}'".format(tag))
-            vcs.gotorevision(tag)
+        if app.UpdateCheckData:
+            filecode, codeex, filever, verex = app.UpdateCheckData.split('|')
 
+            if filecode:
+                filecode = build_dir / filecode
+                if not filecode.is_file():
+                    logging.debug("UpdateCheckData file {0} not found in tag {1}".format(filecode, tag))
+                    continue
+                filecontent = filecode.read_text()
+            else:
+                filecontent = tag
+
+            vercode = tag
+            if codeex:
+                m = re.search(codeex, filecontent)
+                if not m:
+                    continue
+
+                vercode = m.group(1).strip()
+
+            if filever:
+                if filever != '.':
+                    filever = build_dir / filever
+                    if filever.is_file():
+                        filecontent = filever.read_text()
+                    else:
+                        logging.debug("UpdateCheckData file {0} not found in tag {1}".format(filever, tag))
+            else:
+                filecontent = tag
+
+            version = tag
+            if verex:
+                m = re.search(verex, filecontent)
+                if m:
+                    version = m.group(1)
+
+            logging.debug("UpdateCheckData found version {0} ({1})"
+                          .format(version, vercode))
+            i_vercode = common.version_code_string_to_int(vercode)
+            if i_vercode > common.version_code_string_to_int(hcode):
+                htag = tag
+                hcode = str(i_vercode)
+                hver = version
+        else:
             for subdir in possible_subdirs(app):
-                if subdir == '.':
-                    root_dir = build_dir
-                else:
-                    root_dir = os.path.join(build_dir, subdir)
+                root_dir = build_dir / subdir
                 paths = common.manifest_paths(root_dir, last_build.gradle)
-                version, vercode, package = common.parse_androidmanifests(paths, app)
+                version, vercode, _package = common.parse_androidmanifests(paths, app)
+                if version == 'Unknown' or version == 'Ignore':
+                    version = tag
                 if vercode:
                     logging.debug("Manifest exists in subdir '{0}'. Found version {1} ({2})"
                                   .format(subdir, version, vercode))
                     i_vercode = common.version_code_string_to_int(vercode)
                     if i_vercode > common.version_code_string_to_int(hcode):
-                        hpak = package
                         htag = tag
                         hcode = str(i_vercode)
                         hver = version
 
-        if not hpak:
-            return (None, "Couldn't find package ID", None)
-        if hver:
-            return (hver, hcode, htag)
-        return (None, "Couldn't find any version information", None)
-
-    except VCSException as vcse:
-        msg = "VCS error while scanning app {0}: {1}".format(app.id, vcse)
-        return (None, msg, None)
-    except Exception:
-        msg = "Could not scan app {0} due to unknown error: {1}".format(app.id, traceback.format_exc())
-        return (None, msg, None)
+    if hver:
+        if htag != tags[0]:
+            logging.warning(
+                "{appid}: latest tag {tag} does not contain highest version {version}".format(
+                    appid=app.id, tag=tags[0], version=hver
+                )
+            )
+        try:
+            commit = vcs.getref(htag)
+            if commit:
+                return (hver, hcode, commit)
+        except VCSException:
+            pass
+        return (hver, hcode, htag)
+    raise FDroidException(_("Couldn't find any version information"))
 
 
-# Check for a new version by looking at the AndroidManifest.xml at the HEAD
-# of the source repo. Whether this can be used reliably or not depends on
-# the development procedures used by the project's developers. Use it with
-# caution, because it's inappropriate for many projects.
-# Returns (None, "a message") if this didn't work, or (version, vercode) for
-# the details of the current version.
 def check_repomanifest(app, branch=None):
+    """Check for a new version by looking at the AndroidManifest.xml at the HEAD of the source repo.
 
-    try:
+    Whether this can be used reliably or not depends on
+    the development procedures used by the project's developers. Use it with
+    caution, because it's inappropriate for many projects.
+    """
+    if app.RepoType == 'srclib':
+        build_dir = Path('build/srclib') / app.Repo
+        repotype = common.getsrclibvcs(app.Repo)
+    else:
+        build_dir = Path('build') / app.id
+        repotype = app.RepoType
 
-        if app.RepoType == 'srclib':
-            build_dir = os.path.join('build', 'srclib', app.Repo)
-            repotype = common.getsrclibvcs(app.Repo)
-        else:
-            build_dir = os.path.join('build', app.id)
-            repotype = app.RepoType
+    # Set up vcs interface and make sure we have the latest code...
+    vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
 
-        # Set up vcs interface and make sure we have the latest code...
-        vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
+    if repotype == 'git':
+        if branch:
+            branch = 'origin/' + branch
+        vcs.gotorevision(branch)
+    elif repotype == 'git-svn':
+        vcs.gotorevision(branch)
+    elif repotype == 'hg':
+        vcs.gotorevision(branch)
+    elif repotype == 'bzr':
+        vcs.gotorevision(None)
 
-        if repotype == 'git':
-            if branch:
-                branch = 'origin/' + branch
-            vcs.gotorevision(branch)
-        elif repotype == 'git-svn':
-            vcs.gotorevision(branch)
-        elif repotype == 'hg':
-            vcs.gotorevision(branch)
-        elif repotype == 'bzr':
-            vcs.gotorevision(None)
+    last_build = metadata.Build()
+    if app.get('Builds', []):
+        last_build = app.get('Builds', [])[-1]
 
-        last_build = metadata.Build()
-        if len(app.get('Builds', [])) > 0:
-            last_build = app.get('Builds', [])[-1]
+    try_init_submodules(app, last_build, vcs)
 
-        try_init_submodules(app, last_build, vcs)
+    hpak = None
+    hver = None
+    hcode = "0"
+    for subdir in possible_subdirs(app):
+        root_dir = build_dir / subdir
+        paths = common.manifest_paths(root_dir, last_build.gradle)
+        version, vercode, package = common.parse_androidmanifests(paths, app)
+        if vercode:
+            logging.debug("Manifest exists in subdir '{0}'. Found version {1} ({2})"
+                          .format(subdir, version, vercode))
+            i_vercode = common.version_code_string_to_int(vercode)
+            if i_vercode > common.version_code_string_to_int(hcode):
+                hpak = package
+                hcode = str(i_vercode)
+                hver = version
 
-        hpak = None
-        hver = None
-        hcode = "0"
-        for subdir in possible_subdirs(app):
-            if subdir == '.':
-                root_dir = build_dir
-            else:
-                root_dir = os.path.join(build_dir, subdir)
-            paths = common.manifest_paths(root_dir, last_build.gradle)
-            version, vercode, package = common.parse_androidmanifests(paths, app)
-            if vercode:
-                logging.debug("Manifest exists in subdir '{0}'. Found version {1} ({2})"
-                              .format(subdir, version, vercode))
-                if int(vercode) > int(hcode):
-                    hpak = package
-                    hcode = str(int(vercode))
-                    hver = version
-
-        if not hpak:
-            return (None, "Couldn't find package ID")
-        if hver:
-            return (hver, hcode)
-        return (None, "Couldn't find any version information")
-
-    except VCSException as vcse:
-        msg = "VCS error while scanning app {0}: {1}".format(app.id, vcse)
-        return (None, msg)
-    except Exception:
-        msg = "Could not scan app {0} due to unknown error: {1}".format(app.id, traceback.format_exc())
-        return (None, msg)
+    if not hpak:
+        raise FDroidException(_("Couldn't find package ID"))
+    if hver:
+        return (hver, hcode)
+    raise FDroidException(_("Couldn't find any version information"))
 
 
 def check_repotrunk(app):
+    if app.RepoType == 'srclib':
+        build_dir = Path('build/srclib') / app.Repo
+        repotype = common.getsrclibvcs(app.Repo)
+    else:
+        build_dir = Path('build') / app.id
+        repotype = app.RepoType
 
-    try:
-        if app.RepoType == 'srclib':
-            build_dir = os.path.join('build', 'srclib', app.Repo)
-            repotype = common.getsrclibvcs(app.Repo)
-        else:
-            build_dir = os.path.join('build', app.id)
-            repotype = app.RepoType
+    if repotype not in ('git-svn', ):
+        raise MetaDataException(_('RepoTrunk update mode only makes sense in git-svn repositories'))
 
-        if repotype not in ('git-svn', ):
-            return (None, 'RepoTrunk update mode only makes sense in git-svn repositories')
+    # Set up vcs interface and make sure we have the latest code...
+    vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
 
-        # Set up vcs interface and make sure we have the latest code...
-        vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
+    vcs.gotorevision(None)
 
-        vcs.gotorevision(None)
-
-        ref = vcs.getref()
-        return (ref, ref)
-    except VCSException as vcse:
-        msg = "VCS error while scanning app {0}: {1}".format(app.id, vcse)
-        return (None, msg)
-    except Exception:
-        msg = "Could not scan app {0} due to unknown error: {1}".format(app.id, traceback.format_exc())
-        return (None, msg)
+    ref = vcs.getref()
+    return (ref, ref)
 
 
 # Check for a new version by looking at the Google Play Store.
@@ -319,6 +327,7 @@ def check_gplay(app):
 
 def try_init_submodules(app, last_build, vcs):
     """Try to init submodules if the last build entry used them.
+
     They might have been removed from the app's repo in the meantime,
     so if we can't find any submodules we continue with the updates check.
     If there is any other error in initializing them then we stop the check.
@@ -328,15 +337,18 @@ def try_init_submodules(app, last_build, vcs):
             vcs.initsubmodules()
         except NoSubmodulesException:
             logging.info("No submodules present for {}".format(_getappname(app)))
+        except VCSException:
+            logging.info("submodule broken for {}".format(_getappname(app)))
 
 
 # Return all directories under startdir that contain any of the manifest
 # files, and thus are probably an Android project.
 def dirs_with_manifest(startdir):
-    for root, dirs, files in os.walk(startdir):
+    # TODO: Python3.6: Accepts a path-like object.
+    for root, _dirs, files in os.walk(str(startdir)):
         if any(m in files for m in [
                 'AndroidManifest.xml', 'pom.xml', 'build.gradle', 'build.gradle.kts']):
-            yield root
+            yield Path(root)
 
 
 # Tries to find a new subdir starting from the root build_dir. Returns said
@@ -344,9 +356,9 @@ def dirs_with_manifest(startdir):
 def possible_subdirs(app):
 
     if app.RepoType == 'srclib':
-        build_dir = os.path.join('build', 'srclib', app.Repo)
+        build_dir = Path('build/srclib') / app.Repo
     else:
-        build_dir = os.path.join('build', app.id)
+        build_dir = Path('build') / app.id
 
     last_build = app.get_last_build()
 
@@ -354,7 +366,7 @@ def possible_subdirs(app):
         m_paths = common.manifest_paths(d, last_build.gradle)
         package = common.parse_androidmanifests(m_paths, app)[2]
         if package is not None:
-            subdir = os.path.relpath(d, build_dir)
+            subdir = d.relative_to(build_dir)
             logging.debug("Adding possible subdir %s" % subdir)
             yield subdir
 
@@ -374,9 +386,9 @@ def fetch_autoname(app, tag):
         return None
 
     if app.RepoType == 'srclib':
-        build_dir = os.path.join('build', 'srclib', app.Repo)
+        build_dir = Path('build/srclib') / app.Repo
     else:
-        build_dir = os.path.join('build', app.id)
+        build_dir = Path('build') / app.id
 
     try:
         vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
@@ -386,13 +398,10 @@ def fetch_autoname(app, tag):
 
     last_build = app.get_last_build()
 
-    logging.debug("...fetch auto name from " + build_dir)
+    logging.debug("...fetch auto name from " + str(build_dir))
     new_name = None
     for subdir in possible_subdirs(app):
-        if subdir == '.':
-            root_dir = build_dir
-        else:
-            root_dir = os.path.join(build_dir, subdir)
+        root_dir = build_dir / subdir
         new_name = common.fetch_real_name(root_dir, last_build.gradle)
         if new_name is not None:
             break
@@ -416,36 +425,25 @@ def checkupdates_app(app):
     commitmsg = None
 
     tag = None
-    msg = None
     vercode = None
-    noverok = False
     mode = app.UpdateCheckMode
     if mode.startswith('Tags'):
         pattern = mode[5:] if len(mode) > 4 else None
         (version, vercode, tag) = check_tags(app, pattern)
-        if version == 'Unknown':
-            version = tag
-        msg = vercode
     elif mode == 'RepoManifest':
         (version, vercode) = check_repomanifest(app)
-        msg = vercode
     elif mode.startswith('RepoManifest/'):
         tag = mode[13:]
         (version, vercode) = check_repomanifest(app, tag)
-        msg = vercode
     elif mode == 'RepoTrunk':
         (version, vercode) = check_repotrunk(app)
-        msg = vercode
     elif mode == 'HTTP':
         (version, vercode) = check_http(app)
-        msg = vercode
     elif mode in ('None', 'Static'):
-        version = None
-        msg = 'Checking disabled'
-        noverok = True
+        logging.debug('Checking disabled')
+        return
     else:
-        version = None
-        msg = 'Invalid update check method'
+        raise MetaDataException(_('Invalid UpdateCheckMode: {mode}').format(mode=mode))
 
     if version and vercode and app.VercodeOperation:
         if not common.VERCODE_OPERATION_RE.match(app.VercodeOperation):
@@ -464,19 +462,21 @@ def checkupdates_app(app):
 
     updating = False
     if version is None:
-        logmsg = "...{0} : {1}".format(app.id, msg)
-        if noverok:
-            logging.info(logmsg)
-        else:
-            logging.warning(logmsg)
+        raise FDroidException(_('no version information found'))
     elif vercode == app.CurrentVersionCode:
-        logging.info("...up to date")
-    else:
+        logging.debug("...up to date")
+    elif int(vercode) > int(app.CurrentVersionCode):
         logging.debug("...updating - old vercode={0}, new vercode={1}".format(
             app.CurrentVersionCode, vercode))
         app.CurrentVersion = version
         app.CurrentVersionCode = str(int(vercode))
         updating = True
+    else:
+        raise FDroidException(
+            _('current version is newer: old vercode={old}, new vercode={new}').format(
+                old=app.CurrentVersionCode, new=vercode
+            )
+        )
 
     commitmsg = fetch_autoname(app, tag)
 
@@ -489,7 +489,9 @@ def checkupdates_app(app):
     if options.auto:
         mode = app.AutoUpdateMode
         if not app.CurrentVersionCode:
-            logging.warning("Can't auto-update app with no CurrentVersionCode: " + app.id)
+            raise MetaDataException(
+                _("Can't auto-update app with no CurrentVersionCode")
+            )
         elif mode in ('None', 'Static'):
             pass
         elif mode.startswith('Version'):
@@ -510,7 +512,11 @@ def checkupdates_app(app):
                     latest = build
 
             if int(latest.versionCode) > int(app.CurrentVersionCode):
-                logging.info("Refusing to auto update, since the latest build is newer")
+                raise FDroidException(
+                    _(
+                        'latest build recipe is newer: old vercode={old}, new vercode={new}'
+                    ).format(old=latest.versionCode, new=app.CurrentVersionCode)
+                )
 
             if not gotcur:
                 newbuild = copy.deepcopy(latest)
@@ -518,15 +524,21 @@ def checkupdates_app(app):
                 newbuild.versionCode = app.CurrentVersionCode
                 newbuild.versionName = app.CurrentVersion + suffix.replace('%c', newbuild.versionCode)
                 logging.info("...auto-generating build for " + newbuild.versionName)
-                commit = pattern.replace('%v', app.CurrentVersion)
-                commit = commit.replace('%c', newbuild.versionCode)
-                newbuild.commit = commit
+                if tag:
+                    newbuild.commit = tag
+                else:
+                    commit = pattern.replace('%v', app.CurrentVersion)
+                    commit = commit.replace('%c', newbuild.versionCode)
+                    newbuild.commit = commit
+
                 app['Builds'].append(newbuild)
                 name = _getappname(app)
                 ver = _getcvname(app)
                 commitmsg = "Update %s to %s" % (name, ver)
         else:
-            logging.warning('Invalid auto update mode "' + mode + '" on ' + app.id)
+            raise MetaDataException(
+                _('Invalid AutoUpdateMode: {mode}').format(mode=mode)
+            )
 
     if commitmsg:
         metadata.write_metadata(app.metadatapath, app)
@@ -541,8 +553,7 @@ def checkupdates_app(app):
 
 
 def status_update_json(processed, failed):
-    """Output a JSON file with metadata about this run"""
-
+    """Output a JSON file with metadata about this run."""
     logging.debug(_('Outputting JSON'))
     output = common.setup_status_output(start_timestamp)
     if processed:
@@ -552,38 +563,6 @@ def status_update_json(processed, failed):
     common.write_status_json(output)
 
 
-def update_wiki(gplaylog, locallog):
-    if config.get('wiki_server') and config.get('wiki_path'):
-        try:
-            import mwclient
-            site = mwclient.Site((config['wiki_protocol'], config['wiki_server']),
-                                 path=config['wiki_path'])
-            site.login(config['wiki_user'], config['wiki_password'])
-
-            # Write a page with the last build log for this version code
-            wiki_page_path = 'checkupdates_' + time.strftime('%s', start_timestamp)
-            newpage = site.Pages[wiki_page_path]
-            txt = ''
-            txt += "* command line: <code>" + ' '.join(sys.argv) + "</code>\n"
-            txt += common.get_git_describe_link()
-            txt += "* started at " + common.get_wiki_timestamp(start_timestamp) + '\n'
-            txt += "* completed at " + common.get_wiki_timestamp() + '\n'
-            txt += "\n\n"
-            txt += common.get_android_tools_version_log()
-            txt += "\n\n"
-            if gplaylog:
-                txt += '== --gplay check ==\n\n'
-                txt += gplaylog
-            if locallog:
-                txt += '== local source check ==\n\n'
-                txt += locallog
-            newpage.save(txt, summary='Run log')
-            newpage = site.Pages['checkupdates']
-            newpage.save('#REDIRECT [[' + wiki_page_path + ']]', summary='Update redirect')
-        except Exception as e:
-            logging.error(_('Error while attempting to publish log: %s') % e)
-
-
 config = None
 options = None
 start_timestamp = time.gmtime()
@@ -624,10 +603,8 @@ def main():
 
     apps = common.read_app_args(options.appid, allapps, False)
 
-    gplaylog = ''
     if options.gplay:
         for appid, app in apps.items():
-            gplaylog += '* ' + appid + '\n'
             version, reason = check_gplay(app)
             if version is None:
                 if reason == '404':
@@ -649,12 +626,11 @@ def main():
                     else:
                         logging.info("{0} has the same version {1} on the Play Store"
                                      .format(_getappname(app), version))
-        update_wiki(gplaylog, None)
         return
 
-    locallog = ''
     processed = []
     failed = dict()
+    exit_code = 0
     for appid, app in apps.items():
 
         if options.autoonly and app.AutoUpdateMode in ('None', 'Static'):
@@ -663,7 +639,6 @@ def main():
 
         msg = _("Processing {appid}").format(appid=appid)
         logging.info(msg)
-        locallog += '* ' + msg + '\n'
 
         try:
             checkupdates_app(app)
@@ -671,12 +646,12 @@ def main():
         except Exception as e:
             msg = _("...checkupdate failed for {appid} : {error}").format(appid=appid, error=e)
             logging.error(msg)
-            locallog += msg + '\n'
+            logging.debug(traceback.format_exc())
             failed[appid] = str(e)
+            exit_code = 1
 
-    update_wiki(None, locallog)
     status_update_json(processed, failed)
-    logging.info(_("Finished"))
+    sys.exit(exit_code)
 
 
 if __name__ == "__main__":
