@@ -1,5 +1,4 @@
 class FDroidException(Exception):
-
     def __init__(self, value=None, detail=None):
         self.value = value
         self.detail = detail
@@ -9,25 +8,20 @@ class FDroidException(Exception):
             return self.detail
         return '[...]\n' + self.detail[-16000:]
 
-    def get_wikitext(self):
-        ret = repr(self.value) + "\n"
-        if self.detail:
-            ret += "=detail=\n"
-            ret += "<pre>\n" + self.shortened_detail() + "</pre>\n"
-        return ret
-
     def __str__(self):
         if self.value is None:
             ret = __name__
         else:
             ret = str(self.value)
         if self.detail:
-            ret += "\n==== detail begin ====\n%s\n==== detail end ====" % ''.join(self.detail).strip()
+            ret += (
+                "\n==== detail begin ====\n%s\n==== detail end ===="
+                % ''.join(self.detail).strip()
+            )
         return ret
 
 
 class MetaDataException(Exception):
-
     def __init__(self, value):
         self.value = value
 
