@@ -33,8 +33,7 @@ start_timestamp = time.gmtime()
 
 
 def status_update_json(signed):
-    """Output a JSON file with metadata about this run"""
-
+    """Output a JSON file with metadata about this run."""
     logging.debug(_('Outputting JSON'))
     output = common.setup_status_output(start_timestamp)
     if signed:
@@ -60,22 +59,22 @@ def main():
     signed = []
     for output_dir in repodirs:
         if not os.path.isdir(output_dir):
-            raise FDroidException(_("Missing output directory") + " '" + output_dir + "'")
+            raise FDroidException(
+                _("Missing output directory") + " '" + output_dir + "'"
+            )
 
         # Process any apks that are waiting to be signed...
         for f in sorted(glob.glob(os.path.join(output_dir, '*.*'))):
             if common.get_file_extension(f) == 'asc':
                 continue
-            if not common.is_repo_file(f):
+            if not common.is_repo_file(f) and not f.endswith('/index-v1.json'):
                 continue
             filename = os.path.basename(f)
             sigfilename = filename + ".asc"
             sigpath = os.path.join(output_dir, sigfilename)
 
             if not os.path.exists(sigpath):
-                gpgargs = ['gpg', '-a',
-                           '--output', sigpath,
-                           '--detach-sig']
+                gpgargs = ['gpg', '-a', '--output', sigpath, '--detach-sig']
                 if 'gpghome' in config:
                     gpgargs.extend(['--homedir', config['gpghome']])
                 if 'gpgkey' in config:
