@@ -18,9 +18,10 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import os
+import git
+from pathlib import Path
+import platform
 import re
-import glob
 import logging
 import yaml
 try:
@@ -30,9 +31,9 @@ except ImportError:
 import importlib
 from collections import OrderedDict
 
-import fdroidserver.common
-from fdroidserver import _
-from fdroidserver.exception import MetaDataException, FDroidException
+from . import common
+from . import _
+from .exception import MetaDataException, FDroidException
 
 srclibs = None
 warnings_action = None
@@ -43,7 +44,7 @@ VALID_USERNAME_REGEX = re.compile(r'^[a-z\d](?:[a-z\d/._-]){0,38}$', re.IGNORECA
 
 
 def _warn_or_exception(value, cause=None):
-    '''output warning or Exception depending on -W'''
+    """Output warning or Exception depending on -W."""
     if warnings_action == 'ignore':
         pass
     elif warnings_action == 'error':
@@ -89,6 +90,8 @@ yaml_app_field_order = [
     '\n',
     'Builds',
     '\n',
+    'AllowedAPKSigningKeys',
+    '\n',
     'MaintainerNotes',
     '\n',
     'ArchivePolicy',
@@ -144,6 +147,7 @@ class App(dict):
         self.RepoType = ''
         self.Repo = ''
         self.Binaries = None
+        self.AllowedAPKSigningKeys = []
         self.MaintainerNotes = ''
         self.ArchivePolicy = None
         self.AutoUpdateMode = 'None'
@@ -185,8 +189,6 @@ class App(dict):
             return Build()
 
 
-TYPE_UNKNOWN = 0
-TYPE_OBSOLETE = 1
 TYPE_STRING = 2
 TYPE_BOOL = 3
 TYPE_LIST = 4
@@ -200,9 +202,8 @@ fieldtypes = {
     'MaintainerNotes': TYPE_MULTILINE,
     'Categories': TYPE_LIST,
     'AntiFeatures': TYPE_LIST,
+    'AllowedAPKSigningKeys': TYPE_LIST,
     'Build': TYPE_BUILD,
-    'BuildVersion': TYPE_OBSOLETE,
-    'UseBuilt': TYPE_OBSOLETE,
 }
 
 
@@ -325,13 +326,11 @@ class Build(dict):
         return 'ant'
 
     def ndk_path(self):
-        version = self.ndk
-        if not version:
-            version = 'r12b'  # falls back to latest
-        paths = fdroidserver.common.config['ndk_paths']
-        if version not in paths:
-            return ''
-        return paths[version]
+        """Return the path to the first configured NDK or an empty string."""
+        ndk = self.ndk
+        if isinstance(ndk, list):
+            ndk = self.ndk[0]
+        return common.config['ndk_paths'].get(ndk, '')
 
 
 flagtypes = {
@@ -369,8 +368,7 @@ def flagtype(name):
 
 
 class FieldValidator():
-    """
-    Designates App metadata field types and checks that it matches
+    """Designate App metadata field types and checks that it matches.
 
     'name'     - The long name of the field type
     'matching' - List of possible values or regex expression
@@ -439,12 +437,16 @@ valuetypes = {
                    r'^http[s]?://',
                    ["Binaries"]),
 
+    FieldValidator("AllowedAPKSigningKeys",
+                   r'^[a-fA-F0-9]{64}$',
+                   ["AllowedAPKSigningKeys"]),
+
     FieldValidator("Archive Policy",
                    r'^[0-9]+ versions$',
                    ["ArchivePolicy"]),
 
     FieldValidator("Anti-Feature",
-                   r'^(Ads|Tracking|NonFreeNet|NonFreeDep|NonFreeAdd|UpstreamNonFree|NonFreeAssets|KnownVuln|ApplicationDebuggable|NoSourceSince)$',
+                   r'^(Ads|Tracking|NonFreeNet|NonFreeDep|NonFreeAdd|UpstreamNonFree|NonFreeAssets|KnownVuln|ApplicationDebuggable|NoSourceSince|NSFW)$',
                    ["AntiFeatures"]),
 
     FieldValidator("Auto Update Mode",
@@ -471,15 +473,22 @@ def parse_yaml_srclib(metadatapath):
                 'Subdir': None,
                 'Prepare': None}
 
-    if not os.path.exists(metadatapath):
+    if not metadatapath.exists():
         _warn_or_exception(_("Invalid scrlib metadata: '{file}' "
                              "does not exist"
                              .format(file=metadatapath)))
         return thisinfo
 
-    with open(metadatapath, "r", encoding="utf-8") as f:
+    with metadatapath.open("r", encoding="utf-8") as f:
         try:
             data = yaml.load(f, Loader=SafeLoader)
+            if type(data) is not dict:
+                if platform.system() == 'Windows':
+                    # Handle symlink on Windows
+                    symlink = metadatapath.parent / metadatapath.read_text(encoding='utf-8')
+                    if symlink.is_file():
+                        with symlink.open("r", encoding="utf-8") as s:
+                            data = yaml.load(s, Loader=SafeLoader)
             if type(data) is not dict:
                 raise yaml.error.YAMLError(_('{file} is blank or corrupt!')
                                            .format(file=metadatapath))
@@ -487,8 +496,7 @@ def parse_yaml_srclib(metadatapath):
             _warn_or_exception(_("Invalid srclib metadata: could not "
                                  "parse '{file}'")
                                .format(file=metadatapath) + '\n'
-                               + fdroidserver.common.run_yamllint(metadatapath,
-                                                                  indent=4),
+                               + common.run_yamllint(metadatapath, indent=4),
                                cause=e)
             return thisinfo
 
@@ -532,17 +540,15 @@ def read_srclibs():
 
     srclibs = {}
 
-    srcdir = 'srclibs'
-    if not os.path.exists(srcdir):
-        os.makedirs(srcdir)
+    srcdir = Path('srclibs')
+    srcdir.mkdir(exist_ok=True)
 
-    for metadatapath in sorted(glob.glob(os.path.join(srcdir, '*.yml'))):
-        srclibname = os.path.basename(metadatapath[:-4])
-        srclibs[srclibname] = parse_yaml_srclib(metadatapath)
+    for metadatapath in sorted(srcdir.glob('*.yml')):
+        srclibs[metadatapath.stem] = parse_yaml_srclib(metadatapath)
 
 
-def read_metadata(appids={}, refresh=True, sort_by_time=False, check_vcs=False):
-    """Return a list of App instances sorted newest first
+def read_metadata(appids={}, sort_by_time=False):
+    """Return a list of App instances sorted newest first.
 
     This reads all of the metadata files in a 'data' repository, then
     builds a list of App instances from those files.  The list is
@@ -552,7 +558,6 @@ def read_metadata(appids={}, refresh=True, sort_by_time=False, check_vcs=False):
     appids is a dict with appids a keys and versionCodes as values.
 
     """
-
     # Always read the srclibs before the apps, since they can use a srlib as
     # their source repository.
     read_srclibs()
@@ -560,28 +565,17 @@ def read_metadata(appids={}, refresh=True, sort_by_time=False, check_vcs=False):
     apps = OrderedDict()
 
     for basedir in ('metadata', 'tmp'):
-        if not os.path.exists(basedir):
-            os.makedirs(basedir)
+        Path(basedir).mkdir(exist_ok=True)
 
     if appids:
-        vercodes = fdroidserver.common.read_pkg_args(appids)
-        found_invalid = False
-        metadatafiles = []
-        for appid in vercodes.keys():
-            f = os.path.join('metadata', '%s.yml' % appid)
-            if os.path.exists(f):
-                metadatafiles.append(f)
-            else:
-                found_invalid = True
-                logging.critical(_("No such package: %s") % appid)
-        if found_invalid:
-            raise FDroidException(_("Found invalid appids in arguments"))
+        vercodes = common.read_pkg_args(appids)
+        metadatafiles = common.get_metadata_files(vercodes)
     else:
-        metadatafiles = (glob.glob(os.path.join('metadata', '*.yml'))
-                         + glob.glob('.fdroid.yml'))
+        metadatafiles = list(Path('metadata').glob('*.yml')) + list(
+            Path('.').glob('.fdroid.yml'))
 
     if sort_by_time:
-        entries = ((os.stat(path).st_mtime, path) for path in metadatafiles)
+        entries = ((path.stat().st_mtime, path) for path in metadatafiles)
         metadatafiles = []
         for _ignored, path in sorted(entries, reverse=True):
             metadatafiles.append(path)
@@ -590,14 +584,14 @@ def read_metadata(appids={}, refresh=True, sort_by_time=False, check_vcs=False):
         metadatafiles = sorted(metadatafiles)
 
     for metadatapath in metadatafiles:
-        appid, _ignored = fdroidserver.common.get_extension(os.path.basename(metadatapath))
-        if appid != '.fdroid' and not fdroidserver.common.is_valid_package_name(appid):
+        appid = metadatapath.stem
+        if appid != '.fdroid' and not common.is_valid_package_name(appid):
             _warn_or_exception(_("{appid} from {path} is not a valid Java Package Name!")
                                .format(appid=appid, path=metadatapath))
         if appid in apps:
             _warn_or_exception(_("Found multiple metadata files for {appid}")
                                .format(appid=appid))
-        app = parse_metadata(metadatapath, check_vcs, refresh)
+        app = parse_metadata(metadatapath)
         check_metadata(app)
         apps[app.id] = app
 
@@ -695,7 +689,7 @@ def post_metadata_parse(app):
 
 # Parse metadata for a single application.
 #
-#  'metadatapath' - the filename to read. The "Application ID" aka
+#  'metadatapath' - the file path to read. The "Application ID" aka
 #               "Package Name" for the application comes from this
 #               filename. Pass None to get a blank entry.
 #
@@ -730,34 +724,40 @@ def _decode_bool(s):
     _warn_or_exception(_("Invalid boolean '%s'") % s)
 
 
-def parse_metadata(metadatapath, check_vcs=False, refresh=True):
-    '''parse metadata file, optionally checking the git repo for metadata first'''
+def parse_metadata(metadatapath):
+    """Parse metadata file, also checking the source repo for .fdroid.yml.
 
+    If this is a metadata file from fdroiddata, it will first load the
+    source repo type and URL from fdroiddata, then read .fdroid.yml if
+    it exists, then include the rest of the metadata as specified in
+    fdroiddata, so that fdroiddata has precedence over the metadata in
+    the source code.
+
+    """
+    metadatapath = Path(metadatapath)
     app = App()
-    app.metadatapath = metadatapath
-    name, _ignored = fdroidserver.common.get_extension(os.path.basename(metadatapath))
-    if name == '.fdroid':
-        check_vcs = False
-    else:
+    app.metadatapath = metadatapath.as_posix()
+    name = metadatapath.stem
+    if name != '.fdroid':
         app.id = name
 
-    if metadatapath.endswith('.yml'):
-        with open(metadatapath, 'r') as mf:
+    if metadatapath.suffix == '.yml':
+        with metadatapath.open('r', encoding='utf-8') as mf:
             parse_yaml_metadata(mf, app)
     else:
         _warn_or_exception(_('Unknown metadata format: {path} (use: *.yml)')
                            .format(path=metadatapath))
 
-    if check_vcs and app.Repo:
-        build_dir = fdroidserver.common.get_build_dir(app)
-        metadata_in_repo = os.path.join(build_dir, '.fdroid.yml')
-        if not os.path.isfile(metadata_in_repo):
-            vcs, build_dir = fdroidserver.common.setup_vcs(app)
-            if isinstance(vcs, fdroidserver.common.vcs_git):
-                vcs.gotorevision('HEAD', refresh)  # HEAD since we can't know where else to go
-        if os.path.isfile(metadata_in_repo):
-            logging.debug('Including metadata from ' + metadata_in_repo)
-            # do not include fields already provided by main metadata file
+    if metadatapath.name != '.fdroid.yml' and app.Repo:
+        build_dir = common.get_build_dir(app)
+        metadata_in_repo = build_dir / '.fdroid.yml'
+        if metadata_in_repo.is_file():
+            try:
+                # TODO: Python3.6: Should accept path-like
+                commit_id = common.get_head_commit_id(git.Repo(str(build_dir)))
+                logging.debug(_('Including metadata from %s@%s') % (metadata_in_repo, commit_id))
+            except git.exc.InvalidGitRepositoryError:
+                logging.debug(_('Including metadata from {path}').format(metadata_in_repo))
             app_in_repo = parse_metadata(metadata_in_repo)
             for k, v in app_in_repo.items():
                 if k not in app:
@@ -769,35 +769,46 @@ def parse_metadata(metadatapath, check_vcs=False, refresh=True):
         if app.get('Builds'):
             build = app['Builds'][-1]
             if build.subdir:
-                root_dir = build.subdir
+                root_dir = Path(build.subdir)
             else:
-                root_dir = '.'
-            paths = fdroidserver.common.manifest_paths(root_dir, build.gradle)
-            _ignored, _ignored, app.id = fdroidserver.common.parse_androidmanifests(paths, app)
+                root_dir = Path('.')
+            paths = common.manifest_paths(root_dir, build.gradle)
+            _ignored, _ignored, app.id = common.parse_androidmanifests(paths, app)
 
     return app
 
 
 def parse_yaml_metadata(mf, app):
+    """Parse the .yml file and post-process it.
+
+    Clean metadata .yml files can be used directly, but in order to
+    make a better user experience for people editing .yml files, there
+    is post processing.  .fdroid.yml is embedded in the app's source
+    repo, so it is "user-generated".  That means that it can have
+    weird things in it that need to be removed so they don't break the
+    overall process.
+
+    """
     try:
         yamldata = yaml.load(mf, Loader=SafeLoader)
     except yaml.YAMLError as e:
         _warn_or_exception(_("could not parse '{path}'")
                            .format(path=mf.name) + '\n'
-                           + fdroidserver.common.run_yamllint(mf.name,
-                                                              indent=4),
+                           + common.run_yamllint(mf.name, indent=4),
                            cause=e)
 
     deprecated_in_yaml = ['Provides']
 
     if yamldata:
-        for field in yamldata:
-            if field not in yaml_app_fields:
-                if field not in deprecated_in_yaml:
-                    _warn_or_exception(_("Unrecognised app field "
-                                         "'{fieldname}' in '{path}'")
-                                       .format(fieldname=field,
-                                               path=mf.name))
+        for field in tuple(yamldata.keys()):
+            if field not in yaml_app_fields + deprecated_in_yaml:
+                msg = (_("Unrecognised app field '{fieldname}' in '{path}'")
+                       .format(fieldname=field, path=mf.name))
+                if Path(mf.name).name == '.fdroid.yml':
+                    logging.error(msg)
+                    del yamldata[field]
+                else:
+                    _warn_or_exception(msg)
 
         for deprecated_field in deprecated_in_yaml:
             if deprecated_field in yamldata:
@@ -826,7 +837,7 @@ def parse_yaml_metadata(mf, app):
 
 
 def post_parse_yaml_metadata(yamldata):
-    """transform yaml metadata to our internal data format"""
+    """Transform yaml metadata to our internal data format."""
     for build in yamldata.get('Builds', []):
         for flag in build.keys():
             _flagtype = flagtype(flag)
@@ -849,10 +860,13 @@ def post_parse_yaml_metadata(yamldata):
 def write_yaml(mf, app):
     """Write metadata in yaml format.
 
-    :param mf: active file discriptor for writing
-    :param app: app metadata to written to the yaml file
+    Parameters
+    ----------
+    mf
+      active file discriptor for writing
+    app
+      app metadata to written to the yaml file
     """
-
     # import rumael.yaml and check version
     try:
         import ruamel.yaml
@@ -880,10 +894,6 @@ def write_yaml(mf, app):
     _yaml_bools_plus_lists.extend(_yaml_bools_false)
     _yaml_bools_plus_lists.extend([[x] for x in _yaml_bools_false])
 
-    def _class_as_dict_representer(dumper, data):
-        '''Creates a YAML representation of a App/Build instance'''
-        return dumper.represent_dict(data)
-
     def _field_to_yaml(typ, value):
         if typ is TYPE_STRING:
             if value in _yaml_bools_plus_lists:
@@ -925,6 +935,14 @@ def write_yaml(mf, app):
                             cm.update({field: _builds_to_yaml(app)})
                     elif field == 'CurrentVersionCode':
                         cm.update({field: _field_to_yaml(TYPE_INT, getattr(app, field))})
+                    elif field == 'AllowedAPKSigningKeys':
+                        value = getattr(app, field)
+                        if value:
+                            value = [str(i).lower() for i in value]
+                            if len(value) == 1:
+                                cm.update({field: _field_to_yaml(TYPE_STRING, value[0])})
+                            else:
+                                cm.update({field: _field_to_yaml(TYPE_LIST, value)})
                     else:
                         cm.update({field: _field_to_yaml(fieldtype(field), getattr(app, field))})
 
@@ -964,9 +982,12 @@ def write_yaml(mf, app):
         return builds
 
     yaml_app = _app_to_yaml(app)
-    yaml = ruamel.yaml.YAML()
-    yaml.indent(mapping=4, sequence=4, offset=2)
-    yaml.dump(yaml_app, stream=mf)
+    try:
+        yaml = ruamel.yaml.YAML()
+        yaml.indent(mapping=4, sequence=4, offset=2)
+        yaml.dump(yaml_app, stream=mf)
+    except AttributeError:  # Debian/stretch's version does not have YAML()
+        ruamel.yaml.round_trip_dump(yaml_app, mf, indent=4, block_seq_indent=2)
 
 
 build_line_sep = re.compile(r'(?<!\\),')
@@ -974,9 +995,10 @@ build_cont = re.compile(r'^[ \t]')
 
 
 def write_metadata(metadatapath, app):
-    if metadatapath.endswith('.yml'):
+    metadatapath = Path(metadatapath)
+    if metadatapath.suffix == '.yml':
         if importlib.util.find_spec('ruamel.yaml'):
-            with open(metadatapath, 'w') as mf:
+            with metadatapath.open('w') as mf:
                 return write_yaml(mf, app)
         else:
             raise FDroidException(_('ruamel.yaml not installed, can not write metadata.'))
@@ -985,6 +1007,6 @@ def write_metadata(metadatapath, app):
 
 
 def add_metadata_arguments(parser):
-    '''add common command line flags related to metadata processing'''
+    """Add common command line flags related to metadata processing."""
     parser.add_argument("-W", choices=['error', 'warn', 'ignore'], default='error',
                         help=_("force metadata errors (default) to be warnings, or to be ignored."))
