@@ -34,7 +34,7 @@ from .exception import BuildException, VCSException
 config = None
 options = None
 
-DEFAULT_JSON_PER_BUILD = {'errors': [], 'warnings': [], 'infos': []}
+DEFAULT_JSON_PER_BUILD = {'errors': [], 'warnings': [], 'infos': []}  # type: ignore
 json_per_build = DEFAULT_JSON_PER_BUILD
 
 MAVEN_URL_REGEX = re.compile(r"""\smaven\s*{.*?(?:setUrl|url)\s*=?\s*(?:uri)?\(?\s*["']?([^\s"']+)["']?[^}]*}""",
@@ -67,6 +67,8 @@ NON_FREE_GRADLE_LINES = {
         r'google.*admob',
         r'google.*play.*services',
         r'com.google.android.play:core.*',
+        r'com.google.mlkit',
+        r'com.android.billingclient',
         r'androidx.work:work-gcm',
         r'crittercism',
         r'heyzap',
@@ -103,12 +105,19 @@ def get_gradle_compile_commands(build):
 
 
 def scan_binary(apkfile):
-    logging.info("Scanning APK for known non-free classes.")
+    """Scan output of apkanalyzer for known non-free classes.
+
+    apkanalyzer produces useful output when it can run, but it does
+    not support all recent JDK versions, and also some DEX versions,
+    so this cannot count on it to always produce useful output or even
+    to run without exiting with an error.
+
+    """
+    logging.info(_('Scanning APK with apkanalyzer for known non-free classes.'))
     result = common.SdkToolsPopen(["apkanalyzer", "dex", "packages", "--defined-only", apkfile], output=False)
-    problems = 0
     if result.returncode != 0:
-        problems += 1
-        logging.error(result.output)
+        logging.warning(_('scanner not cleanly run apkanalyzer: %s') % result.output)
+    problems = 0
     for suspect, regexp in CODE_SIGNATURES.items():
         matches = regexp.findall(result.output)
         if matches:
@@ -121,24 +130,26 @@ def scan_binary(apkfile):
 
 
 def scan_source(build_dir, build=metadata.Build()):
-    """Scan the source code in the given directory (and all subdirectories)
-    and return the number of fatal problems encountered
-    """
+    """Scan the source code in the given directory (and all subdirectories).
 
+    Returns
+    -------
+    the number of fatal problems encountered.
+    """
     count = 0
 
-    whitelisted = [
+    allowlisted = [
         'firebase-jobdispatcher',  # https://github.com/firebase/firebase-jobdispatcher-android/blob/master/LICENSE
         'com.firebaseui',          # https://github.com/firebase/FirebaseUI-Android/blob/master/LICENSE
         'geofire-android'          # https://github.com/firebase/geofire-java/blob/master/LICENSE
     ]
 
-    def is_whitelisted(s):
-        return any(wl in s for wl in whitelisted)
+    def is_allowlisted(s):
+        return any(al in s for al in allowlisted)
 
     def suspects_found(s):
         for n, r in NON_FREE_GRADLE_LINES.items():
-            if r.match(s) and not is_whitelisted(s):
+            if r.match(s) and not is_allowlisted(s):
                 yield n
 
     allowed_repos = [re.compile(r'^https://' + re.escape(repo) + r'/*') for repo in [
@@ -184,10 +195,18 @@ def scan_source(build_dir, build=metadata.Build()):
         return False
 
     def ignoreproblem(what, path_in_build_dir):
-        """
-        :param what: string describing the problem, will be printed in log messages
-        :param path_in_build_dir: path to the file relative to `build`-dir
-        "returns: 0 as we explicitly ignore the file, so don't count an error
+        """No summary.
+
+        Parameters
+        ----------
+        what: string
+          describing the problem, will be printed in log messages
+        path_in_build_dir
+          path to the file relative to `build`-dir
+
+        Returns
+        -------
+        0 as we explicitly ignore the file, so don't count an error
         """
         msg = ('Ignoring %s at %s' % (what, path_in_build_dir))
         logging.info(msg)
@@ -196,11 +215,20 @@ def scan_source(build_dir, build=metadata.Build()):
         return 0
 
     def removeproblem(what, path_in_build_dir, filepath):
-        """
-        :param what: string describing the problem, will be printed in log messages
-        :param path_in_build_dir: path to the file relative to `build`-dir
-        :param filepath: Path (relative to our current path) to the file
-        "returns: 0 as we deleted the offending file
+        """No summary.
+
+        Parameters
+        ----------
+        what: string
+          describing the problem, will be printed in log messages
+        path_in_build_dir
+          path to the file relative to `build`-dir
+        filepath
+          Path (relative to our current path) to the file
+
+        Returns
+        -------
+        0 as we deleted the offending file
         """
         msg = ('Removing %s at %s' % (what, path_in_build_dir))
         logging.info(msg)
@@ -216,10 +244,18 @@ def scan_source(build_dir, build=metadata.Build()):
         return 0
 
     def warnproblem(what, path_in_build_dir):
-        """
-        :param what: string describing the problem, will be printed in log messages
-        :param path_in_build_dir: path to the file relative to `build`-dir
-        :returns: 0, as warnings don't count as errors
+        """No summary.
+
+        Parameters
+        ----------
+        what: string
+          describing the problem, will be printed in log messages
+        path_in_build_dir
+          path to the file relative to `build`-dir
+
+        Returns
+        -------
+        0, as warnings don't count as errors
         """
         if toignore(path_in_build_dir):
             return 0
@@ -229,13 +265,22 @@ def scan_source(build_dir, build=metadata.Build()):
         return 0
 
     def handleproblem(what, path_in_build_dir, filepath):
-        """Dispatches to problem handlers (ignore, delete, warn) or returns 1
-         for increasing the error count
+        """Dispatches to problem handlers (ignore, delete, warn).
 
-        :param what: string describing the problem, will be printed in log messages
-        :param path_in_build_dir: path to the file relative to `build`-dir
-        :param filepath: Path (relative to our current path) to the file
-        :returns: 0 if the problem was ignored/deleted/is only a warning, 1 otherwise
+        Or returns 1 for increasing the error count.
+
+        Parameters
+        ----------
+        what: string
+          describing the problem, will be printed in log messages
+        path_in_build_dir
+          path to the file relative to `build`-dir
+        filepath
+          Path (relative to our current path) to the file
+
+        Returns
+        -------
+        0 if the problem was ignored/deleted/is only a warning, 1 otherwise
         """
         if toignore(path_in_build_dir):
             return ignoreproblem(what, path_in_build_dir)
@@ -302,33 +347,33 @@ def scan_source(build_dir, build=metadata.Build()):
                 continue
 
             path_in_build_dir = os.path.relpath(filepath, build_dir)
-            extension = os.path.splitext(path_in_build_dir)[1]
 
             if curfile in ('gradle-wrapper.jar', 'gradlew', 'gradlew.bat'):
                 removeproblem(curfile, path_in_build_dir, filepath)
-            elif extension == '.apk':
+            elif curfile.endswith('.apk'):
                 removeproblem(_('Android APK file'), path_in_build_dir, filepath)
 
-            elif extension == '.a':
+            elif curfile.endswith('.a'):
                 count += handleproblem(_('static library'), path_in_build_dir, filepath)
-            elif extension == '.aar':
+            elif curfile.endswith('.aar'):
                 count += handleproblem(_('Android AAR library'), path_in_build_dir, filepath)
-            elif extension == '.class':
+            elif curfile.endswith('.class'):
                 count += handleproblem(_('Java compiled class'), path_in_build_dir, filepath)
-            elif extension == '.dex':
+            elif curfile.endswith('.dex'):
                 count += handleproblem(_('Android DEX code'), path_in_build_dir, filepath)
-            elif extension == '.gz':
+            elif curfile.endswith('.gz'):
                 count += handleproblem(_('gzip file archive'), path_in_build_dir, filepath)
-            elif extension == '.so':
+            # We use a regular expression here to also match versioned shared objects like .so.0.0.0
+            elif re.match(r'.*\.so(\..+)*$', curfile):
                 count += handleproblem(_('shared library'), path_in_build_dir, filepath)
-            elif extension == '.zip':
+            elif curfile.endswith('.zip'):
                 count += handleproblem(_('ZIP file archive'), path_in_build_dir, filepath)
-            elif extension == '.jar':
+            elif curfile.endswith('.jar'):
                 for name in suspects_found(curfile):
                     count += handleproblem('usual suspect \'%s\'' % name, path_in_build_dir, filepath)
                 count += handleproblem(_('Java JAR file'), path_in_build_dir, filepath)
 
-            elif extension == '.java':
+            elif curfile.endswith('.java'):
                 if not os.path.isfile(filepath):
                     continue
                 with open(filepath, 'r', errors='replace') as f:
@@ -337,7 +382,7 @@ def scan_source(build_dir, build=metadata.Build()):
                             count += handleproblem('DexClassLoader', path_in_build_dir, filepath)
                             break
 
-            elif extension == '.gradle':
+            elif curfile.endswith('.gradle'):
                 if not os.path.isfile(filepath):
                     continue
                 with open(filepath, 'r', errors='replace') as f:
@@ -353,7 +398,7 @@ def scan_source(build_dir, build=metadata.Build()):
                     if not any(r.match(url) for r in allowed_repos):
                         count += handleproblem('unknown maven repo \'%s\'' % url, path_in_build_dir, filepath)
 
-            elif extension in ['', '.bin', '.out', '.exe']:
+            elif curfile.endswith(('.', '.bin', '.out', '.exe')):
                 if is_binary(filepath):
                     count += handleproblem('binary', path_in_build_dir, filepath)
 
