@@ -95,6 +95,13 @@ if os.getenv('CI_PROJECT_NAMESPACE') != 'fdroid':
 git_repo = git.repo.Repo('.')
 modified = git_repo.git().ls_files(modified=True).split()
 if git_repo.is_dirty() and ('gradlew-fdroid' in modified or 'makebuildserver' in modified):
+    private_token = os.getenv('PERSONAL_ACCESS_TOKEN')
+    if not private_token:
+        print(Fore.RED
+              + 'ERROR: GitLab Token not found in PERSONAL_ACCESS_TOKEN!'
+              + Style.RESET_ALL)
+        exit(1)
+
     branch = git_repo.create_head(os.path.basename(__file__), force=True)
     branch.checkout()
     git_repo.index.add(['gradlew-fdroid', 'makebuildserver'])
@@ -112,12 +119,6 @@ if git_repo.is_dirty() and ('gradlew-fdroid' in modified or 'makebuildserver' in
     remote.push(force=True)
     git.remote.Remote.rm(git_repo, remote_name)
 
-    private_token = os.getenv('PERSONAL_ACCESS_TOKEN')
-    if not private_token:
-        print(Fore.RED
-              + 'ERROR: GitLab Token not found in PERSONAL_ACCESS_TOKEN!'
-              + Style.RESET_ALL)
-        exit(1)
     gl = gitlab.Gitlab(os.getenv('CI_SERVER_URL'), api_version=4,
                        private_token=private_token)
     project = gl.projects.get(project_path, lazy=True)
@@ -126,13 +127,19 @@ if git_repo.is_dirty() and ('gradlew-fdroid' in modified or 'makebuildserver' in
         '\n\n<p><small>generated by <a href="%s/-/jobs/%s">GitLab CI Job #%s</a></small></p>'
         % (os.getenv('CI_PROJECT_URL'), os.getenv('CI_JOB_ID'), os.getenv('CI_JOB_ID'))
     )
-    mr = project.mergerequests.create({
-        'source_branch': branch.name,
-        'target_project_id': 36527,  # fdroid/fdroidserver
-        'target_branch': 'master',
-        'title': 'update to gradle v' + version,
-        'description': description,
-        'labels': ['fdroid-bot', 'gradle'],
-        'remove_source_branch': True,
-    })
-    mr.save()
+    try:
+        mr = project.mergerequests.create({
+            'source_branch': branch.name,
+            'target_project_id': 36527,  # fdroid/fdroidserver
+            'target_branch': 'master',
+            'title': 'update to gradle v' + version,
+            'description': description,
+            'labels': ['fdroid-bot', 'gradle'],
+            'remove_source_branch': True,
+        })
+        mr.save()
+    except gitlab.exceptions.GitlabCreateError as e:
+        if e.response_code == 409:  # Another open merge request already exists for this source branch
+            print(e.error_message)
+        else:
+            raise e
